function oModel = model( varargin )
% MODEL model class constructor.
% s = model(tModelName,vk,vX0,tOutputExpression,vlegend,sT0,tODEFunction)
%==========================================================================
% tModelName        =>name of the model
% vk                => nominal parameter vector, to be used for default
%                   cases.
% vX0               => nominal initial condition vector
% tOutputExpression => expression consisting of state variables to be used
%                   as output. Is is a string.
% vlegend           => vector of strings. Names of various chemical species
% involved.
% sT0               => expected period of oscillations, default value is 24.
% tODEFunction      => type of MATLAB ODE function to be used, default is
%                   ode45.It is a string.
%
%==========================================================================
% First three parameters are required to create the objects. Others are
% optional. All the parameters must appear in the given order.
% Objects can also be created by passing an object of type MODEL to
% this function.

rModel.mReactantsComment    = 'Stochiometric matrix with the reactants :';
rModel.mProductsComment     = 'Stochiometric matrix with the products :';
rModel.slkComment           = 'No. of parameters      :';                                % comments for the 'display' function
rModel.slXComment           = 'No. of state variables :';
rModel.tModelNameComment    = 'Model name             :';
rModel.sT0Comment           = 'Expected period        :';
rModel.vkComment            = 'Parameters             :';
rModel.tODEFunctionComment  = 'ODE function used      :';
rModel.tPathComment         = 'Folder location        :';
rModel.vX0Comment           = 'Initial conditions     :';
rModel.tOutputExpComment    = 'Output                 :';

SelectWorkingPath;

switch (nargin)
    case 0
        
        rModel.tModelName = 'input_test';% model_name
        rModel.vk = [1 1];% nominal_parameters
        rModel.vX0 = [1 1];% nominal_initialcondition
        rModel.tOutputExpression = 'X1';% output---OPTIONAL
        rModel.vlegend = {'a','b'};% legend---OPTIONAL
        rModel.sT0 = 1;% expected_period---OPTIONAL
        rModel.tODEFunction = 'ode45';% ODE function to be used for integration---OPTIONAL
        rModel.slk = 2;% Number of parameters
        rModel.slX = 2;% Number of state variables
        rModel.slr = 2;% Number of reactions
        rModel.sconstrained = 0; % Number of mass constraints
        rModel.tPath = sprintf('%s/models/test',bioSimworkingpath);% Folder location
        rModel.unitvolume = []; % Unit for the cell volume
        rModel.unitconcentration = [];% Unit for the concentration
        oModel = class(rModel,'model');
        
        
    case 1
        %       If a single argument of class model, return it
        if (isa(varargin{1},'model'));
            oModel = varargin{1};
        else
            error('Input argument is not an object of class model');
        end


    case 3

        for i=1:length(varargin{3})
            temp=sprintf('X%i',i);
            legend_default{i}=temp;
        end

        %       check for correct input arguments

        if (~ischar(varargin{1}))
            error('Incorrect first input argument. See HELP');
        elseif(~isnumeric(varargin{2}))
            error('Incorrect second input argument. See HELP');
        elseif(~isnumeric(varargin{3}))
            error('Incorrect third input argument. See HELP');
        end
        %       Create object using specified values, keep default for rest

        rModel.tModelName = varargin{1};% model_name
        rModel.vk = varargin{2};% nominal_parameters
        rModel.vX0 = varargin{3};% nominal_initialcondition
        rModel.tOutputExpression = '';% output---OPTIONAL
        rModel.vlegend = legend_default;% legend---OPTIONAL
        rModel.sT0 = 24;% expected_period---OPTIONAL
        rModel.tODEFunction = 'ode45';% ODE function to be used for integration---OPTIONAL
        rModel.slk = length(varargin{2});% Number of parameters
        rModel.slX = length(varargin{3});% Number of state variables
        rModel.slr = [];% Number of reactions
        rModel.sconstrained = 0; % Number of mass constraints
        rModel.tPath = sprintf('%s/models/%s',bioSimworkingpath, varargin{1});% Folder location
        rModel.unitvolume = []; % Unit for the cell volume
        rModel.unitconcentration = [];% Unit for the concentration
        oModel = class(rModel,'model');

    case 4

        for i=1:length(varargin{3})
            temp=sprintf('X%i',i);
            legend_default{i}=temp;
        end

        %       check for correct input arguments

        if (~ischar(varargin{1}))
            error('Incorrect first input argument. See HELP');
        elseif(~isnumeric(varargin{2}))
            error('Incorrect second input argument. See HELP');
        elseif(~isnumeric(varargin{3}))
            error('Incorrect third input argument. See HELP');
        elseif(~ischar(varargin{4}))
            error('Incorrect fourth input argument. See HELP');
        end
        %       Create object using specified values, keep default for rest

        rModel.tModelName = varargin{1};% model_name
        rModel.vk = varargin{2};% nominal_parameters
        rModel.vX0 = varargin{3};% nominal_initialcondition
        rModel.tOutputExpression =varargin{4};% output---OPTIONAL
        rModel.vlegend =legend_default;% legend---OPTIONAL
        rModel.sT0 =[];% expected_period---OPTIONAL
        rModel.tODEFunction = 'ode45';% ODE function to be used for integration---OPTIONAL
        rModel.slk = length(varargin{2});% Number of parameters
        rModel.slX = length(varargin{3});% Number of state variables
        rModel.slr = [];% Number of reactions
        rModel.sconstrained = 0; % Number of mass constraints
        rModel.tPath = sprintf('%s/models/%s',bioSimworkingpath, varargin{1});% Folder location
        rModel.unitvolume = []; % Unit for the cell volume
        rModel.unitconcentration = [];% Unit for the concentration
        oModel = class(rModel,'model');

    case 5

        for i=1:length(varargin{3})
            temp=sprintf('X%i',i);
            legend_default{i}=temp;
        end


        %       check for correct input arguments

        if (~ischar(varargin{1}))
            error('Incorrect first input argument. See HELP');
        elseif(~isnumeric(varargin{2}))
            error('Incorrect second input argument. See HELP');
        elseif(~isnumeric(varargin{3}))
            error('Incorrect third input argument. See HELP');
        elseif(~ischar(varargin{4}))
            error('Incorrect fourth input argument. See HELP');
            %         elseif(~ischar(varargin{5}))
            %             error('Incorrect fifth input argument. See HELP');
        end
        %       Create object using specified values, keep default for rest

        rModel.tModelName = varargin{1};% model_name
        rModel.vk = varargin{2};% nominal_parameters
        rModel.vX0 = varargin{3};% nominal_initialcondition
        rModel.tOutputExpression =varargin{4};% output---OPTIONAL
        rModel.vlegend =varargin{5};% legend---OPTIONAL
        rModel.sT0 =24;% expected_period---OPTIONAL
        rModel.tODEFunction = 'ode45';% ODE function to be used for integration---OPTIONAL
        rModel.slk = length(varargin{2});% Number of parameters
        rModel.slX = length(varargin{3});% Number of state variables
        rModel.slr = [];% Number of reactions
        rModel.sconstrained = 0; % Number of mass constraints
        rModel.tPath = sprintf('%s/models/%s',bioSimworkingpath, varargin{1});% Folder location
        rModel.unitvolume = []; % Unit for the cell volume
        rModel.unitconcentration = [];% Unit for the concentration
        oModel = class(rModel,'model');

    case 6

        for i=1:length(varargin{3})
            temp=sprintf('X%i',i);
            legend_default{i}=temp;
        end


        %       check for correct input arguments

        if (~ischar(varargin{1}))
            error('Incorrect first input argument. See HELP');
        elseif(~isnumeric(varargin{2}))
            error('Incorrect second input argument. See HELP');
        elseif(~isnumeric(varargin{3}))
            error('Incorrect third input argument. See HELP');
        elseif(~ischar(varargin{4}))
            error('Incorrect fourth input argument. See HELP');
            %         elseif(~ischar(varargin{5}))
            %             error('Incorrect fifth input argument. See HELP');
        end
        %       Create object using specified values, keep default for rest

        rModel.tModelName = varargin{1};% model_name
        rModel.vk = varargin{2};% nominal_parameters
        rModel.vX0 = varargin{3};% nominal_initialcondition
        rModel.tOutputExpression =varargin{4};% output---OPTIONAL
        rModel.vlegend =varargin{5};% legend---OPTIONAL
        rModel.sT0 =varargin{6};% expected_period---OPTIONAL
        rModel.tODEFunction = 'ode45';% ODE function to be used for integration---OPTIONAL
        rModel.slk = length(varargin{2});% Number of parameters
        rModel.slX = length(varargin{3});% Number of state variables
        rModel.slr = [];% Number of reactions
        rModel.sconstrained = 0; % Number of mass constraints
        rModel.tPath = sprintf('%s/models/%s',bioSimworkingpath, varargin{1});% Folder location
        rModel.unitvolume = []; % Unit for the cell volume
        rModel.unitconcentration = [];% Unit for the concentration
        oModel = class(rModel,'model');

    case 7

        for i=1:length(varargin{3})
            temp=sprintf('X%i',i);
            legend_default{i}=temp;
        end


        %       check for correct input arguments

        if (~ischar(varargin{1}))
            error('Incorrect first input argument. See HELP');
        elseif(~isnumeric(varargin{2}))
            error('Incorrect second input argument. See HELP');
        elseif(~isnumeric(varargin{3}))
            error('Incorrect third input argument. See HELP');
        elseif(~ischar(varargin{4}))
            error('Incorrect fourth input argument. See HELP');
            %         elseif(~ischar(varargin{5}))
            %             error('Incorrect fifth input argument. See HELP');
        end
        %       Create object using specified values, keep default for rest

        rModel.tModelName = varargin{1};% model_name
        rModel.vk = varargin{2};% nominal_parameters
        rModel.vX0 = varargin{3};% nominal_initialcondition
        rModel.tOutputExpression =varargin{4};% output---OPTIONAL
        rModel.vlegend =varargin{5};% legend---OPTIONAL
        rModel.sT0 =varargin{6};% expected_period---OPTIONAL
        rModel.tODEFunction = varargin{7};% ODE function to be used for integration---OPTIONAL
        rModel.slk = length(varargin{2});% Number of parameters
        rModel.slX = length(varargin{3});% Number of state variables
        rModel.slr = [];% Number of reactions
        rModel.sconstrained = 0; % Number of mass constraints
        rModel.tPath = sprintf('%s/models/%s',bioSimworkingpath, varargin{1});% Folder location
        rModel.unitvolume = []; % Unit for the cell volume
        rModel.unitconcentration = [];% Unit for the concentration
        oModel = class(rModel,'model');

    otherwise
        error ('Invalid number of arguments. See HELP');

end


if ~strcmp(rModel.tModelName,'input_test')

    SelectWorkingPath
    % make the folder corresponding to each model
    tPath = [bioSimworkingpath '/models/'];
    f1=fullfile(tPath,rModel.tModelName);
    if (exist(f1,'dir')==0)
        mkdir(f1);
    end

    createFrameIntegrators(rModel);

    % adding the Path the folder to matlab path
    eval (sprintf('addpath %s;',oModel.tPath));

end

end
